﻿#pragma once

#include <string>
#include "..\Common\DeviceResources.h"
#include "..\Common\StepTimer.h"

namespace PlatonicSolids
{
    class PlatonicSolidsRenderer
    {
    public:
        PlatonicSolidsRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void Update(DX::StepTimer const& timer);
        void Render();

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;

        struct Triangle3D
        {
            DirectX::XMFLOAT4 point1;
            DirectX::XMFLOAT4 point2;
            DirectX::XMFLOAT4 point3;
        };

        struct FaceDefinition
        {
            std::vector<DirectX::XMFLOAT3> vertices;
        };

        struct PolyhedronDefinition
        {
            std::vector<FaceDefinition> faces;
        };

        struct RenderInfo
        {
            Microsoft::WRL::ComPtr<ID2D1Mesh> mesh;
            Microsoft::WRL::ComPtr<ID2D1SolidColorBrush> brush;
        };

        struct FigureInfo
        {
            // Constructor
            FigureInfo()
            {
            }

            // Move constructor 
            FigureInfo(FigureInfo && other) : 
                srcTriangles(std::move(other.srcTriangles)),
                dstTriangles(std::move(other.dstTriangles)),
                renderInfo(std::move(other.renderInfo))
            {
            }

            int faceCount;
            int trianglesPerFace;
            std::vector<Triangle3D> srcTriangles;
            std::vector<Triangle3D> dstTriangles;
            D2D1_COLOR_F color;
            std::vector<RenderInfo> renderInfo;
        };

        std::vector<FigureInfo> m_figureInfos;
        
        void GenerateTriangles(PolyhedronDefinition& polyDef, std::vector<Triangle3D>& triangles);
    };
}